﻿using System;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using C1.C1Report;
using System.Collections.ObjectModel;
using System.Data;
using System.Collections.Generic;

namespace MailMerging.Web
{
    [ServiceContract(Namespace = "")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class C1ReportService
    {
        /// <summary>
        /// Get a list of the reports available on the server.
        /// </summary>
        [OperationContract]
        public string[] GetReportList()
        {
            var fn = GetReportDefinitionFile("mailmerge.xml");

            return C1Report.GetReportList(fn);
        }
        /// <summary>
        /// Get a PDF stream containing the specified report.
        /// </summary>
        ///
        static string GetReportDefinitionFile(string filename)
        {
            return Path.Combine(AppDomain.CurrentDomain.BaseDirectory, filename);
        }



        [OperationContract]
        public string[] GetCalculatedFields(string reportname)
        {
            var fields = new List<string>();
            var c1r = new C1Report();
            var fn = GetReportDefinitionFile("mailmerge.xml");

            c1r.Load(fn, reportname);

            foreach (Field field in c1r.Sections["Detail"].Fields)
            {
                if (field.Calculated == true)
                {
                    if (!field.Text.Contains("+"))
                        fields.Add(field.Text);
                    else
                    {
                        var list = field.Text.Split(new char[] { ' ' }).ToList().Where(a => a.StartsWith("+") && a.Length > 1).Select(a => a);
                        foreach (var f in list)
                        {

                            var t = System.Text.RegularExpressions.Regex.Matches(f, @"\+.*?\+", System.Text.RegularExpressions.RegexOptions.IgnorePatternWhitespace);
                            if (t.Count != 0)
                            {
                                fields.Add(t[0].Value.Trim(new char[] { '+' }));
                            }

                        }
                    }

                }
            }

            return fields.Distinct().ToArray();
        }

        [OperationContract]
        public byte[] GetReport(string reportName)
        {
            // make sure the requested report exists
            if (!GetReportList().Contains(reportName))
            {
                throw new Exception("The report requested was not found in the report definition file.");
            }

            // load report
            var c1r = new C1Report();
            var fn = GetReportDefinitionFile("mailmerge.xml");

            c1r.Load(fn, reportName);

            // export to pdf
            using (var ms = new MemoryStream())
            {
                c1r.RenderToStream(ms, FileFormatEnum.PDF);
                return ms.ToArray();
            }
        }

  

        [OperationContract]
        public byte[] GetReportWithCustomDataSource(string reportName, byte[] reportsource)
        {
            var _report = new C1.C1Report.C1Report();
            var fn = GetReportDefinitionFile("mailmerge.xml");
            _report.Load(fn, reportName);
            var ds = new DataSet();
            ds.ReadXml(new MemoryStream(reportsource));
            _report.DataSource.Recordset = ds.Tables[0];

            using (var ms = new MemoryStream())
            {
                _report.RenderToStream(ms, FileFormatEnum.PDF);
                return ms.ToArray();
            }

        }




    }
}
